#
# Unit Tests for coopr.dae.misc
#

import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__))+os.sep

import coopr.environ
from coopr.pyomo import *
from coopr.dae import *
from coopr.dae.misc import *
import pyutilib.th as unittest

class TestDaeMisc(unittest.TestCase):
    
    # test generate_finite_elements method
    def test_generate_finite_elements(self):
        m = ConcreteModel()
        m.t = DifferentialSet(bounds=(0,10))
        m.t2 = DifferentialSet(bounds=(0,10))
        m.t3 = DifferentialSet(bounds=(0,1))

        oldt = sorted(m.t)
        generate_finite_elements(m.t,1)
        self.assertTrue(oldt == sorted(m.t))
        self.assertFalse(m.t.get_changed())
        generate_finite_elements(m.t,2)
        self.assertFalse(oldt == sorted(m.t))
        self.assertTrue(m.t.get_changed())
        self.assertTrue([0,5.0,10] == sorted(m.t))
        generate_finite_elements(m.t,3)
        self.assertTrue([0,2.5,5.0,10] == sorted(m.t))
        generate_finite_elements(m.t,5)
        self.assertTrue([0,1.25,2.5,5.0,7.5,10] == sorted(m.t))

        generate_finite_elements(m.t2,10)
        self.assertTrue(len(m.t2) == 11)
        self.assertTrue([0,1,2,3,4,5,6,7,8,9,10] == sorted(m.t2))

        generate_finite_elements(m.t3,7)
        self.assertTrue(len(m.t3) == 8)
        t = sorted(m.t3)
        self.assertTrue(t[1] == 0.14285714285714285)
      
    # test generate_collocation_points method
    def test_generate_collocation_points(self):
        m = ConcreteModel()
        m.t = DifferentialSet(initialize=[0,1])
        m.t2 = DifferentialSet(initialize=[0,2,4,6])
        
        radau = True
        tau1 = [1]
        oldt = sorted(m.t)
        generate_colloc_points(m.t,tau1,radau)
        self.assertTrue(oldt == sorted(m.t))
        self.assertFalse(m.t.get_changed())

        radau = False
        tau1 = [0.5]
        oldt = sorted(m.t)
        generate_colloc_points(m.t,tau1,radau)
        self.assertFalse(oldt == sorted(m.t))
        self.assertTrue(m.t.get_changed())
        self.assertTrue([0,0.5,1] == sorted(m.t))

        radau = True
        tau2 = [0.2,0.3,0.7,0.8,1]
        generate_colloc_points(m.t,tau2,radau)
        self.assertTrue(len(m.t) == 11)
        self.assertTrue([0,0.1,0.15,0.35,0.4,0.5,0.6,0.65,0.85,0.9,1] == sorted(m.t))

        generate_colloc_points(m.t2,tau2,radau)
        self.assertTrue(len(m.t2) == 16)
        self.assertTrue(m.t2.get_changed())
        t = sorted(m.t2)
        self.assertTrue(t[1] == 0.4)
        self.assertTrue(t[13] == 5.4)

    # test Params indexed only by a differentialset after discretizing
    def test_discretized_params_single(self):
        m = ConcreteModel()
        m.t = DifferentialSet(bounds=(0,10))
        m.s1 = Set(initialize=[1,2,3])
        m.s2 = Set(initialize=[(1,1),(2,2)])
        m.p1 = Param(m.t, initialize=1)
        m.p2 = Param(m.t, default=2)
        m.p3 = Param(m.t, initialize=1, default=2)
        def _rule1(m,i):
            return i**2
        def _rule2(m,i):
            return 2*i
        m.p4 = Param(m.t, initialize={0:5,10:5}, default=_rule1)
        m.p5 = Param(m.t, initialize=_rule1, default=_rule2)

        generate_finite_elements(m.t,5)
        
        try:
            for i in m.t:
                m.p1[i]
            self.fail("Expected ValueError because no default value "\
                          "was specified")
        except ValueError:
            pass

        for i in m.t:
            self.assertEqual(m.p2[i], 2)

            if i == 0 or i == 10:
                self.assertEqual(m.p3[i],1)
                self.assertEqual(m.p4[i],5)
                self.assertEqual(m.p5[i],i**2)
            else:
                self.assertEqual(m.p3[i],2)
                self.assertEqual(m.p4[i],i**2)
                self.assertEqual(m.p5[i], 2*i)

    # test Params with multiple indexing sets after discretizing
    def test_discretized_params_multiple(self):
        m = ConcreteModel()
        m.t = DifferentialSet(bounds=(0,10))
        m.s1 = Set(initialize=[1,2,3])
        m.s2 = Set(initialize=[(1,1),(2,2)])
        def _rule1(m,i):
            return i**2
        m.p1 = Param(m.s1,m.t,initialize=2,default=_rule1)
        m.p2 = Param(m.t,m.s1,default=5)
        def _rule2(m,i,j):
            return i+j
        m.p3 = Param(m.s1,m.t,initialize=2,default=_rule2)
        def _rule3(m,i,j,k):
            return i+j+k
        m.p4 = Param(m.s2,m.t,default=_rule3)

        generate_finite_elements(m.t,5)

        try:
            for i in m.p1.keys():
                m.p1[i]
            self.fail("Expected TypeError because a function with the "\
                          "wrong number of arguments was specified as "\
                          "the default")
        except TypeError:
            pass

        for i in m.p2.keys():
            self.assertEqual(m.p2[i],5)

        for i in m.t:
            for j in m.s1:
                if i==0 or i==10:
                    self.assertEqual(m.p3[j,i],2)
                else:
                    self.assertEqual(m.p3[j,i],i+j)

        for i in m.t:
            for j in m.s2:
                self.assertEqual(m.p4[j,i],sum(j,i))
        
        
    # test update_diffset_indexed_component method for Vars with 
    # single index of the differentialset
    def test_update_diffset_indexed_component_vars_single(self):
        m = ConcreteModel()
        m.t = DifferentialSet(bounds=(0,10))
        m.t2 = DifferentialSet(initialize=[1,2,3])
        m.s = Set(initialize=[1,2,3])
        m.v1 = Var(m.t,initialize=3)
        m.v2 = Var(m.t,bounds=(4,10),initialize={0:2,10:12})
        def _init(m,i):
            return i
        m.v3 = Var(m.t,bounds=(-5,5),initialize=_init)
        m.v4 = Var(m.s,initialize=7)
        m.v5 = Var(m.t2)

        generate_finite_elements(m.t,5)
        update_diffset_indexed_component(m.v1)
        update_diffset_indexed_component(m.v2)
        update_diffset_indexed_component(m.v3)
        update_diffset_indexed_component(m.v4)
        update_diffset_indexed_component(m.v5)

        self.assertTrue(len(m.v1)==6)
        self.assertTrue(len(m.v2)==6)
        self.assertTrue(len(m.v3)==6)
        self.assertTrue(len(m.v4)==3)
        self.assertTrue(len(m.v5)==3)

        self.assertTrue(value(m.v1[2])==3)
        self.assertTrue(m.v1[4].ub is None)
        self.assertTrue(m.v1[6].lb is None)
        
        self.assertTrue(m.v2[2].value is None)
        self.assertTrue(m.v2[4].lb == 4)
        self.assertTrue(m.v2[8].ub == 10)
        self.assertTrue(value(m.v2[0])==2)

        self.assertTrue(value(m.v3[2]) == 2)
        self.assertTrue(m.v3[4].lb == -5)
        self.assertTrue(m.v3[6].ub == 5)
        self.assertTrue(value(m.v3[8]) == 8)

    # test update_diffset_indexed_component method for Vars with 
    # multiple indices
    def test_update_diffset_indexed_component_vars_multiple(self):
        m = ConcreteModel()
        m.t = DifferentialSet(bounds=(0,10))
        m.t2 = DifferentialSet(initialize=[1,2,3])
        m.s = Set(initialize=[1,2,3])
        m.s2 = Set(initialize=[(1,1),(2,2)])
        m.v1 = Var(m.s,m.t,initialize=3)
        m.v2 = Var(m.s,m.t,m.t2,bounds=(4,10),initialize={(1,0,1):22,(2,10,2):22})
        def _init(m,i,j,k):
            return i
        m.v3 = Var(m.t,m.s2,bounds=(-5,5),initialize=_init)
        m.v4 = Var(m.s,m.t2,initialize=7)

        generate_finite_elements(m.t,5)
        update_diffset_indexed_component(m.v1)
        update_diffset_indexed_component(m.v2)
        update_diffset_indexed_component(m.v3)
        update_diffset_indexed_component(m.v4)

        self.assertTrue(len(m.v1)==18)
        self.assertTrue(len(m.v2)==54)
        self.assertTrue(len(m.v3)==12)
        self.assertTrue(len(m.v4)==9)

        self.assertTrue(value(m.v1[1,4])==3)
        self.assertTrue(m.v1[2,2].ub is None)
        self.assertTrue(m.v1[3,8].lb is None)
        
        self.assertTrue(value(m.v2[1,0,1]) == 22)
        self.assertTrue(m.v2[1,2,1].value is None)
        self.assertTrue(m.v2[2,4,3].lb == 4)
        self.assertTrue(m.v2[3,8,1].ub == 10)

        self.assertTrue(value(m.v3[2,2,2])==2)
        self.assertTrue(m.v3[4,1,1].lb == -5)
        self.assertTrue(m.v3[8,2,2].ub == 5)
        self.assertTrue(value(m.v3[6,1,1]) == 6)

    # test update_diffset_indexed_component method for Constraints with
    # single index of the differentialset
    def test_update_diffset_indexed_component_constraints_single(self):
        m = ConcreteModel()
        m.t = DifferentialSet(bounds=(0,10))
        m.p = Param(m.t,default=3)
        m.v = Var(m.t, initialize=5)
        
        def _con1(m,i):
            return m.p[i]*m.v[i] <= 20
        m.con1 = Constraint(m.t, rule=_con1)
        
        # Rules that iterate over a differentialset implicitly are not updated
        # after the discretization
        def _con2(m):
            return sum(m.v[i] for i in m.t) >= 0
        m.con2 = Constraint(rule=_con2)

        generate_finite_elements(m.t,5)
        update_diffset_indexed_component(m.v)
        update_diffset_indexed_component(m.p)
        update_diffset_indexed_component(m.con1)
        update_diffset_indexed_component(m.con2)

        self.assertTrue(len(m.con1)==6)
        self.assertEqual(m.con1[2](), 15)
        self.assertEqual(m.con1[8](), 15)
        self.assertEqual(m.con2(), 10)

    # test update_diffset_indexed_component method for Constraints with
    # multiple indices
    def test_update_diffset_indexed_component_constraints_multiple(self):
        m = ConcreteModel()
        m.t = DifferentialSet(bounds=(0,10))
        m.t2 = DifferentialSet(initialize=[1,2,3])
        m.s1 = Set(initialize=[1,2,3])
        m.s2 = Set(initialize=[(1,1),(2,2)])
        def _init(m,i,j):
            return j+i
        m.p1 = Param(m.s1,m.t,default=_init)
        m.v1 = Var(m.s1, m.t, initialize=5)
        m.v2 = Var(m.s2,m.t,initialize=2)
        m.v3 = Var(m.t2,m.s2, initialize=1)

        def _con1(m,si,ti):
            return m.v1[si,ti]*m.p1[si,ti] >= 0
        m.con1 = Constraint(m.s1,m.t,rule=_con1)

        def _con2(m,i,j,ti):
            return m.v2[i,j,ti]+m.p1[1,ti] == 10
        m.con2 = Constraint(m.s2,m.t,rule=_con2)

        def _con3(m,i,ti,ti2,j,k):
            return m.v1[i,ti]-m.v3[ti2,j,k]*m.p1[i,ti] <= 20
        m.con3 = Constraint(m.s1,m.t,m.t2,m.s2,rule=_con3)

        generate_finite_elements(m.t,5)
        update_diffset_indexed_component(m.p1)
        update_diffset_indexed_component(m.v1)
        update_diffset_indexed_component(m.v2)
        update_diffset_indexed_component(m.v3)
        update_diffset_indexed_component(m.con1)
        update_diffset_indexed_component(m.con2)
        update_diffset_indexed_component(m.con3)
        
        self.assertTrue(len(m.con1) == 18)
        self.assertTrue(len(m.con2) == 12)
        self.assertTrue(len(m.con3) == 108)

        self.assertEqual(m.con1[1,4](), 25)
        self.assertEqual(m.con1[2,6](), 40)
        self.assertEqual(m.con1[3,8](), 55)
        self.assertTrue(value(m.con1[2,4].lower) == 0)
        self.assertTrue(value(m.con1[1,8].upper) == None)

        self.assertEqual(m.con2[1,1,2](), 5)
        self.assertEqual(m.con2[2,2,4](), 7)
        self.assertEqual(m.con2[1,1,8](), 11)
        self.assertTrue(value(m.con2[2,2,6].lower) == 10)
        self.assertTrue(value(m.con2[1,1,10].upper) == 10)
        
        self.assertEqual(m.con3[1,2,1,1,1](), 2)
        self.assertEqual(m.con3[1,4,1,2,2](), 0)
        self.assertEqual(m.con3[2,6,3,1,1](), -3)
        self.assertEqual(m.con3[3,8,2,2,2](), -6)
        self.assertTrue(value(m.con3[2,0,2,1,1].lower) == None)
        self.assertTrue(value(m.con3[3,2,3,2,2].upper) == 20)                   
 
    # test update_diffset_indexed_component method for other components
    def test_update_diffset_indexed_component_other(self):
        m = ConcreteModel()
        m.t = DifferentialSet(bounds=(0,10))
        m.junk = Suffix()
        m.s = Set(initialize=[1,2,3])
        m.v = Var(m.s)
        def _obj(m):
            return sum(m.v[i] for i in m.s)
        m.obj = Objective(rule=_obj)

        generate_finite_elements(m.t,5)
        update_diffset_indexed_component(m.junk)
        update_diffset_indexed_component(m.s)
        update_diffset_indexed_component(m.obj)
        
    # test add_equality_constraints method
    def test_add_equality_constraints(self):
        m = ConcreteModel()
        m.t = DifferentialSet(initialize=[1,2,3,4,5])
        m.s = Set(initialize=[1,2])
        m.v = Var(m.t)
        m.v2 = Var(m.s,m.t,initialize=2)
        
        def _vdot(m,i):
            return m.v[i]**2
        m.vdot = Differential(dv=m.v,rule=_vdot)

        def _vdot2(m,i,j):
            return m.v2[i,j]**2
        m.vdot2 = Differential(dv=m.v2,rule=_vdot2)

        self.assertTrue(len(m.vdot._cons)==0)
        add_equality_constraints(m.vdot)
        self.assertTrue(len(m.vdot._cons)==4)
        m.vdot._lhs_var[2] = 3
        m.v[2] = 3
        self.assertEqual(m.vdot._cons[1](),-6)

        add_equality_constraints(m.vdot2)
        self.assertTrue(len(m.vdot2._cons)==8)
        m.vdot2._lhs_var[1,2] = 3
        self.assertEqual(m.vdot2._cons[1](),-1)

    # test add_equality_constraints method with invalid 
    # Differential rules
    def test_bad_add_equality_constraint(self):
        m = ConcreteModel()
        m.t = DifferentialSet(initialize=[1,2,3])
        m.v = Var(m.t)

        def _vdot1(m,i,j):
            return m.v[i,j]**2
        m.vdot1 = Differential(dv=m.v,rule=_vdot1)

        def _vdot2(m,i):
            return m.v[i] == 3
        m.vdot2 = Differential(dv=m.v,rule=_vdot2)

        try:
            add_equality_constraints(m.vdot1)
            self.fail("Expected TypeError because the rule supplied to the differential "\
                          "had the wrong number of arguments")
        except TypeError:
            pass

        try:
            add_equality_constraints(m.vdot2)
            self.fail("Expected TypeError because the rule supplied to the differential "\
                          "returns an invalid expression")
        except TypeError:
            pass

if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['DifferentialSet']

import logging

from coopr.pyomo import *
from coopr.pyomo.base.sets import OrderedSimpleSet
from coopr.pyomo.base.numvalue import native_numeric_types

logger = logging.getLogger('coopr.pyomo')


class DifferentialSet(OrderedSimpleSet):
    """ DifferentialSet objects are used to index DAE Pyomo objects
    
        This is a set that represents a list of numeric values. Minimally, 
        this set must contain two values defining a range over which a differential
        equations will be discretized and solved. If additional values are specified
        they will be used as finite elements locations in the discretization. 

        Constructor
            Arguments:
                name        The name of the differential set
                rule
                initialize  Default set members, which may be overridden
                            when setting up this set
                doc         Documentation for this object
        Private Attributes:
            _changed   This keeps track of whether or not the diffset was
                       changed during discretization. If the user specifies 
                       all of the needed discretization points before the 
                       discretization then there is no need to go back through
                       the model and reconstruct things indexed by the diffset
    """
    def __init__(self, *args, **kwds):
        """ Constructor """
        if kwds.pop("filter", None) is not None:
            raise TypeError("'filter' is not a valid keyword argument for DifferentialSet")
        if kwds.pop("within", None) is not None:
            raise TypeError("'within' is not a valid keyword argument for DifferentialSet")
        if kwds.pop("dimen", None) is not None:
            raise TypeError("'dimen' is not a valid keyword argument for DifferentialSet")
        if kwds.pop("virtual", None) is not None:
            raise TypeError("'virtual' is not a valid keyword argument for DifferentialSet")
        if kwds.pop("validate", None) is not None:
            raise TypeError("'validate' is not a valid keyword argument for DifferentialSet")
        if len(args) != 0:
            raise TypeError("A DifferentialSet expects no arguments")
        
        kwds.setdefault('ctype',DifferentialSet)
        kwds.setdefault('ordered',Set.SortedOrder)
        self._type=DifferentialSet
        self._changed = False
        self.concrete=True
        self.virtual=False
        self._fe = []
        self._discretization_info = {}
        OrderedSimpleSet.__init__(self, **kwds)

    def get_finite_elements(self):
        return self._fe

    def get_discretization_info(self):
        return self._discretization_info

    def get_changed(self):
        return self._changed

    def set_changed(self,value):
        if value is not True and value is not False:
            raise ValueError("The _changed attribute on a DifferentialSet may only "\
                "be set to True or False")
        self._changed = value

    def get_upper_element_boundary(self, value):
        """
        This function returns the first finite element that is greater 
        or equal to the value sent to the function
        """
        if value in self._fe:
            return value
        elif value > max(self._fe):
            print("****WARNING: The value '%s' exceeds the upper bound "\
                "of the diffset '%s'. Returning the upper bound" %(str(value),self.name))
            return max(self._fe)
        else :
            for i in self._fe:
                if i>value:     # This works because the list _fe is always sorted
                    return i

    def get_lower_element_boundary(self, value):
        """
        This function returns the first finite element that is less than 
        or equal to the value sent to the function
        """
        if value in self._fe:
            if self._discretization_info.has_key('scheme'):
                if self._discretization_info['scheme'] == 'Radau Collocation':
                    # Because Radau Collocation has a collocation point on the 
                    # upper finite element bound this if statement ensures that
                    # the desired finite element bound is returned
                    tmp = self._fe.index(value)
                    if tmp != 0:
                        return self._fe[tmp-1]
            return value 
        elif value < min(self._fe):
            print("****WARNING: The value '%s' is less than the lower bound "\
                "of the diffset '%s'. Returning the lower bound" %(str(value),self.name))
            return min(self._fe)
        else:
            rev_fe = list(self._fe)
            rev_fe.reverse()
            for i in rev_fe:
                if i < value:
                    return i

    def construct(self, values=None):
        OrderedSimpleSet.construct(self, values)
         
        for val in self.value:
            if type(val) is tuple:
                raise ValueError("DifferentialSet cannot contain tuples")
            if val.__class__ not in native_numeric_types:
                raise ValueError("DifferentialSet can only contain numeric values")

        if self._bounds is None:
            # This is needed because if a set is initialized using a function
            # the bounds are not updated automatically
            #if self.dimen == 1 and len(self>2):
            #    self._bounds[0] == min(self)
            #    self._bounds[1] == max(self)
            raise ValueError("DifferentialSet '%s' must have at least two values indicating "\
                "the range over which a differential equation is to be discretized" % (self.name))
        if self._bounds[0].__class__ not in native_numeric_types:
            raise ValueError("Bounds on DifferentialSet must be numeric values")
        if self._bounds[1].__class__ not in native_numeric_types:
            raise ValueError("Bounds on DifferentialSet must be numeric values")

        # TBD: If a user specifies bounds they will be added to the set
        # unless the user specified bounds have been overwritten during 
        # OrderedSimpleSet construction. This can lead to some unintuitive
        # behavior when the diffset is both initialized with values and
        # bounds are specified. The current implementation is consistent 
        # with how 'Set' treats this situation. 
        if self._bounds[0] not in self.value:
            self.add(self._bounds[0])
        if self._bounds[1] not in self.value:
            self.add(self._bounds[1])
        
        if len(self) < 2:
            raise ValueError("DifferentialSet '%s' must have at least two values indicating "\
                "the range over which a differential equation is to be discretized" % (self.name))
        self._fe = sorted(self)


register_component(DifferentialSet, "A sequence of numeric values defining the range over which a differential equation is to be discretized and solved.")



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

import logging
import sys

from coopr.core.plugin import alias
from coopr.pyomo.base import Transformation
from coopr.pyomo import *
from coopr.pyomo.base import Block
from coopr.dae import *
from coopr.dae.misc import generate_finite_elements
from coopr.dae.misc import update_diffset_indexed_component
from coopr.dae.misc import add_equality_constraints


logger = logging.getLogger('coopr.pyomo')


class Implicit_Euler_Transformation(Transformation):

    alias('dae.euler_discretization', doc="TODO")

    def __init__(self):
        super(Implicit_Euler_Transformation, self).__init__()

    def _setup(self, instance):
        instance = instance.clone()
        instance.concrete_mode()
        self._nfe = {}
        self._blocks = list(instance.all_blocks())
        return instance

    def apply(self, instance, **kwds):
        instance = self._setup(instance)  
        tmpnfe = kwds.pop('nfe',10)
        tmpds = kwds.pop('diffset',None)

        if tmpds is not None and tmpds.type() is not DifferentialSet:
            raise TypeError("The component specified using the 'diffset' keyword "\
                "must be a differential set")

        if self._nfe.has_key(None):
            raise ValueError("A general discretization scheme has already been applied to "\
                    "to every differential set in the model. If you would like to specify a "\
                    "specific discretization scheme for one of the differential sets you must discretize "\
                    "each differential set individually. If you would like to apply a different "\
                    "discretization scheme to all differential sets you must declare a new Implicit_"\
                    "Euler object")

        if len(self._nfe) == 0 and tmpds is None:
            # Same discretization on all differentialsets
            self._nfe[None] = tmpnfe
            currentds = None
        elif self._nfe.has_key(tmpds.name):
            raise ValueError("A discretization scheme has already been specified "\
                    "for DifferentialSet '%s'" %s(tmpds.name))
        else :
            self._nfe[tmpds.name]=tmpnfe
            currentds = tmpds.name

        # Note: all_blocks is a generator and since we are changing the
        # type of Differential components to Block after the discretization
        # we have to explicitly form the list of all Blocks before iterating
        # over them. Otherwise, the differential components will be included
        # in the iteration and _transformBlock will be called unintentionally 
        # for each differential component after their type has been changed.
        for block in self._blocks:
            print(block)
            self._transformBlock(block,currentds)

        # Taken from bigm
        # REQUIRED: re-call preprocess()
        instance.preprocess()
        return instance

    def _transformBlock(self, block,currentds):

        self._fe = {}
        for ds in block.components(DifferentialSet).itervalues():
            if currentds is None or currentds is ds.name:
                generate_finite_elements(ds,self._nfe[currentds])
                if not ds.get_changed():
                    if len(ds)-1 > self._nfe[currentds]:
                        print("***WARNING: More finite elements were found in DifferentialSet "\
                            "'%s' than the number of finite elements specified in apply. "\
                            "The larger number of finite elements will be used." %s(ds.name))
                
                self._nfe[ds.name]=len(ds)-1
                self._fe[ds.name]=sorted(ds)
                ds._fe=sorted(ds)
                # Adding discretization information to the differentialset object itself
                # so that it can be accessed outside of the discretization object
                disc_info = ds.get_discretization_info()
                disc_info['nfe']=self._nfe[ds.name]
                disc_info['scheme']='Implicit Euler'
                     
        # Maybe check to see if any of the DifferentialSets have been changed,
        # if they haven't then the model components need not be updated
        # or even iterated through

        for c in block.components().itervalues():
            if c.type() != Differential:
                update_diffset_indexed_component(c)
            else:
                update_diffset_indexed_component(c._lhs_var)
        
        for d in block.components(Differential).itervalues():
            if currentds is None or d.get_differentialset().name is currentds:
                add_equality_constraints(d)
                self._impEuler_transform(d,)
                block.reclassify_subcomponent_type(d,Block)
            
    def _impEuler_transform(self,diff):
        # generate the constraints unique to this discretization scheme
        # The needed constraint is:
        # 1) y[t_k+1] == y[t_k] + (t_k+1 - t_k)*y_dot[t_k+1]
        t = sorted(diff.get_differentialset())
        l = diff._ds_argindex
        dv = diff.get_diffvar()
        if None in diff._non_ds:
            tmpidx = (None,)
        elif len(diff._non_ds)==1:
            tmpidx = diff._non_ds[0]
        else:
            tmpidx = diff._non_ds[0].cross(*diff._non_ds[1:])

        for i in tmpidx:
            for k in range(len(t)-1):
                if i is None:
                    idx1 = t[k]
                    idx2 = t[k+1]
                else:
                    tmpi=i
                    if not isinstance(i,tuple):
                        tmpi = (i,)
                    idx1 = tmpi[0:l]+(t[k],)+tmpi[l:]
                    idx2 = tmpi[0:l]+(t[k+1],)+tmpi[l:]

                diff._cons.add(dv[idx2]==dv[idx1]+(t[k+1]-t[k])*diff[idx2])



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ( 'DerivativeVar', 'InputVar', 'StateVar', )

import weakref

from coopr.pyomo.base.component import ComponentData, register_component
from coopr.pyomo.base.numvalue import NumericValue
from coopr.pyomo.base.sparse_indexed_component import SparseIndexedComponent
from coopr.dae.diffset import DifferentialSet

class _DifferentialVarData(ComponentData, NumericValue):

    __slots__ = ()

    def __init__(self, owner):
        """Constructor"""
        ComponentData.__init__(self, owner)

    def __getstate__(self):
        state = super(_DifferentialVarData, self).__getstate__()
        for i in _DifferentialVarData.__slots__:
            state[i] = getattr(self, i)
        return state

    # Note: because NONE of the slots on this class need to be edited,
    # we don't need to implement a specialized __setstate__ method, and
    # can quietly rely on the super() class's implementation.
    # def __setstate__(self, state):
    #     pass

    def is_constant(self):
        return False

    def is_fixed(self):
        return False

    def __nonzero__(self):
        """Return True if the value is defined and non-zero."""
        return True

    __bool__ = __nonzero__

    def __call__(self, exception=True):
        """Return the value of this object."""
        raise RuntimeError("_DifferentialVarData has not been flushed out yet:"
                           " value() not defined.")

    def pprint(self, ostream=None, verbose=False):
        if ostream is None:         #pragma:nocover
            ostream = sys.stdout
        print >>ostream, str(self),


class DifferentialVar(SparseIndexedComponent):

    def __init__(self, *args, **kwds):
        #
        self._diffset = kwds.pop("diffset", None)
        if self._diffset is None:
            for i,s in enumerate(args):
                if s.__class__ is DifferentialSet:
                    self._diffset = s
                    args = args[:i] + args[i+1:]
                    self._diffIndex = i
                    break
        else:
            self._diffIndex = 0
        if self._diffset is None:
            raise ValueError("InputVar must be indexed by a DifferentialSet")
        SparseIndexedComponent.__init__(self, *args, **kwds)

    def _default(self, idx):
        return self._data.setdefault(idx, _DifferentialVarData(self))

    def __getitem__(self, ndx):
        try:
            return super(DifferentialVar, self).__getitem__(ndx)
        except KeyError:
            pass
        idx = ndx[:self._diffIndex] + ndx[self._diffIndex+1:]
        return super(DifferentialVar, self).\
            __getitem__(idx)[ndx[self._diffIndex]]



class StateVar(DifferentialVar):
    def __new__(cls, *args, **kwds):
        if cls != StateVar:
            return super(StateVar, cls).__new__(cls)
        largs = len(args)
        if 'diffset' not in kwds:
            largs -= 1
        if largs == 0:
            return _ScalarStateVar.__new__(_ScalarStateVar)
        else:
            return _IndexedStateVar.__new__(_IndexedStateVar)

    def __init__(self, *args, **kwds):
        self._derivative = None
        kwds.setdefault('ctype', StateVar)
        DifferentialVar.__init__(self, *args, **kwds)

    def derivative(self):
        return self._derivative

class _ScalarStateVar(_DifferentialVarData, StateVar):
    def __init__(self, *args, **kwds):
        StateVar.__init__(self, *args, **kwds)
        _DifferentialVarData.__init__(self, self)

class _IndexedStateVar(StateVar):
    pass



class InputVar(DifferentialVar):
    def __new__(cls, *args, **kwds):
        if cls != InputVar:
            return super(InputVar, cls).__new__(cls)
        largs = len(args)
        if 'diffset' not in kwds:
            largs -= 1
        if largs == 0:
            return _ScalarInputVar.__new__(_ScalarInputVar)
        else:
            return _IndexedInputVar.__new__(_IndexedInputVar)

    def __init__(self, *args, **kwds):
        kwds.setdefault('ctype', InputVar)
        DifferentialVar.__init__(self, *args, **kwds)

class _ScalarInputVar(_DifferentialVarData, InputVar):
    def __init__(self, *args, **kwds):
        InputVar.__init__(self, *args, **kwds)
        _DifferentialVarData.__init__(self, self)

class _IndexedInputVar(InputVar):
    pass



class DerivativeVar(DifferentialVar):
    def __new__(cls, *args, **kwds):
        if cls != DerivativeVar:
            return super(DerivativeVar, cls).__new__(cls)
        if len(args) != 1:
            raise ValueError(
                "DerivativeVar must be created with a single argument: "
                "the StateVar that this is the derivative of")
        if args[0].is_indexed():
            return _IndexedDerivativeVar.__new__(_IndexedDerivativeVar)
        else:
            return _ScalarDerivativeVar.__new__(_ScalarDerivativeVar)

    def __init__(self, stateVar, **kwds):
        if stateVar.derivative() is not None:
            raise ValueError(
                "Cannot create a new derivative variable for State variable "
                "%s: derivative already defined as %s" 
                % ( stateVar.cname(True), stateVar.derivative().cname(True) ) )
        stateVar._derivative = weakref.ref(self)
        self._stateVar = stateVar
        kwds['diffset'] = stateVar._diffset
        kwds.setdefault('ctype', DerivativeVar)
        if stateVar.is_indexed():
            DifferentialVar.__init__(self, stateVar.index_set(), **kwds)
        else:
            DifferentialVar.__init__(self, **kwds)
        self._diffIndex = stateVar._diffIndex

class _ScalarDerivativeVar(_DifferentialVarData, DerivativeVar):
    def __init__(self, *args, **kwds):
        DerivativeVar.__init__(self, *args, **kwds)
        _DifferentialVarData.__init__(self, self)

class _IndexedDerivativeVar(DerivativeVar):
    pass


register_component(StateVar, "State variable in a DAE model.")
register_component(InputVar, "Input or Control variable in a DAE model.")
register_component(DerivativeVar, "Derivative of a State variable in a DAE model.")




# 
# Unit Tests for DifferentialSet() Objects
#

import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__))+os.sep

import coopr.environ
from coopr.pyomo import *
from coopr.dae import *
import pyutilib.th as unittest

class TestDifferentialSet(unittest.TestCase):

    # test __init__
    def test_init(self):
        model = ConcreteModel()
        model.t = DifferentialSet(bounds=(0,1))
        del model.t

        model.t = DifferentialSet(initialize=[1,2,3])
        del model.t
    
        model.t = DifferentialSet(bounds=(0,5),initialize=[1,3,5])
        del model.t

        try:
            model.t = DifferentialSet()
            self.fail("Expected ValueError because a DifferentialSet component"/
                      " must contain at least two values upon construction")
        except ValueError:
            pass

    # test bad keyword arguments
    def test_bad_kwds(self):
        model = ConcreteModel()
        try:
            model.t = DifferentialSet(bounds=(0,1),filter=True)
            self.fail("Expected TypeError")
        except TypeError:
            pass
        
        try:
            model.t = DifferentialSet(bounds=(0,1),within=PositiveReals)
            self.fail("Expected TypeError")
        except TypeError:
            pass
        
        try:
            model.t = DifferentialSet(bounds=(0,1),dimen=2)
            self.fail("Expected TypeError")
        except TypeError:
            pass
        
        try:
            model.t = DifferentialSet(bounds=(0,1),virtual=True)
            self.fail("Expected TypeError")
        except TypeError:
            pass

        try:
            model.t = DifferentialSet(bounds=(0,1),validate=True)
            self.fail("Expected TypeError")
        except TypeError:
            pass

    # test valid declarations
    def test_valid_declaration(self):
        model = ConcreteModel()
        model.t = DifferentialSet(bounds=(0,1))
        self.assertTrue(len(model.t)==2)
        self.assertTrue(0 in model.t)
        self.assertTrue(1 in model.t)
        del model.t

        model.t = DifferentialSet(initialize=[1,2,3])
        self.assertTrue(len(model.t)==3)
        self.assertTrue(model.t.first()==1)
        self.assertTrue(model.t.last()==3)
        del model.t
        
        model.t = DifferentialSet(bounds=(0,4),initialize=[1,2,3])
        self.assertTrue(len(model.t)==5)
        self.assertTrue(model.t.first()==0)
        self.assertTrue(model.t.last()==4)
        del model.t

        model.t = DifferentialSet(bounds=(0,4),initialize=[1,2,3,5])
        self.assertTrue(len(model.t)==5)
        self.assertTrue(model.t.first()==0)
        self.assertTrue(model.t.last()==5)
        self.assertTrue(4 not in model.t)
        del model.t

        model.t = DifferentialSet(bounds=(2,6),initialize=[1,2,3,5])
        self.assertTrue(len(model.t)==5)
        self.assertTrue(model.t.first()==1)
        self.assertTrue(model.t.last()==6)
        del model.t

        model.t = DifferentialSet(bounds=(2,4),initialize=[1,3,5])
        self.assertTrue(len(model.t)==3)
        self.assertTrue(2 not in model.t)
        self.assertTrue(4 not in model.t)

    # test invalid declarations
    def test_invalid_declaration(self):
        model = ConcreteModel()
        model.s = Set(initialize=[1,2,3])

        try:
            model.t = DifferentialSet(model.s,bounds=(0,1))
            self.fail("Expected TypeError")
        except TypeError:
            pass

        try:
            model.t = DifferentialSet(bounds=(0,0))
            self.fail("Expected ValueError")
        except ValueError:
            pass

        try:
            model.t = DifferentialSet(initialize=[1])
            self.fail("Expected ValueError")
        except ValueError:
            pass

        try:
            model.t = DifferentialSet(bounds=(None,1))
            self.fail("Expected ValueError")
        except ValueError:
            pass

        try:
            model.t = DifferentialSet(bounds=(0,None))
            self.fail("Expected ValueError")
        except ValueError:
            pass

        try:
            model.t = DifferentialSet(initialize=[(1,2),(3,4)])
            self.fail("Expected ValueError")
        except ValueError:
            pass

        try:
            model.t = DifferentialSet(initialize=['foo','bar'])
            self.fail("Expected ValueError")
        except ValueError:
            pass

    # test the get_changed method
    def test_get_changed(self):
        model = ConcreteModel()
        model.t = DifferentialSet(initialize=[1,2,3])
        self.assertFalse(model.t.get_changed())
        self.assertEqual(model.t._changed,model.t.get_changed())

    # test the set_changed method
    def test_set_changed(self):
        model = ConcreteModel()
        model.t = DifferentialSet(initialize=[1,2,3])
        self.assertFalse(model.t._changed)
        model.t.set_changed(True)
        self.assertTrue(model.t._changed)
        model.t.set_changed(False)
        self.assertFalse(model.t._changed)

        try:
            model.t.set_changed(3)
            self.fail("Expected a ValueError")
        except ValueError:
            pass

class TestIO(unittest.TestCase):
    
    def setUp(self):
        #
        # Create Model
        #
        self.model = AbstractModel()
    
    def tearDown(self):
        if os.path.exists("diffset.dat"):
            os.remove("diffset.dat")

    def test_io1(self):
        OUTPUT=open("diffset.dat","w")
        OUTPUT.write("data;\n")
        OUTPUT.write("set A := 1 3 5 7;\n")
        OUTPUT.write("end;\n")
        OUTPUT.close()
        self.model.A = DifferentialSet()
        self.instance = self.model.create("diffset.dat")
        self.assertEqual( len(self.instance.A), 4 )

    def test_io2(self):
        OUTPUT=open("diffset.dat","w")
        OUTPUT.write("data;\n")
        OUTPUT.write("set A := 1 3 5;\n")
        OUTPUT.write("end;\n")
        OUTPUT.close()
        self.model.A = DifferentialSet(bounds=(0,4))
        self.instance = self.model.create("diffset.dat")
        self.assertEqual( len(self.instance.A), 4)

    def test_io3(self):
        OUTPUT=open("diffset.dat","w")
        OUTPUT.write("data;\n")
        OUTPUT.write("set A := 1 3 5;\n")
        OUTPUT.write("end;\n")
        OUTPUT.close()
        self.model.A = DifferentialSet(bounds=(2,6))
        self.instance = self.model.create("diffset.dat")
        self.assertEqual( len(self.instance.A), 4)

    def test_io4(self):
        OUTPUT=open("diffset.dat","w")
        OUTPUT.write("data;\n")
        OUTPUT.write("set A := 1 3 5;\n")
        OUTPUT.write("end;\n")
        OUTPUT.close()
        self.model.A = DifferentialSet(bounds=(2,4))
        self.instance = self.model.create("diffset.dat")
        self.assertEqual( len(self.instance.A), 3)
    
    def test_io5(self):
        OUTPUT=open("diffset.dat","w")
        OUTPUT.write("data;\n")
        OUTPUT.write("set A := 1 3 5;\n")
        OUTPUT.write("end;\n")
        OUTPUT.close()
        self.model.A = DifferentialSet(bounds=(0,6))
        self.instance = self.model.create("diffset.dat")
        self.assertEqual( len(self.instance.A), 5)

    def test_io6(self):
        OUTPUT=open("diffset.dat","w")
        OUTPUT.write("data;\n")
        OUTPUT.write("set B := 1;\n")
        OUTPUT.write("end;\n")
        OUTPUT.close()
        self.model.B = DifferentialSet()
        try:
            self.instance = self.model.create("diffset.dat")
            self.fail("Expected ValueError because data set has only one value"\
                          " and no bounds are specified")
        except ValueError:
            pass
            
    def test_io7(self):
        OUTPUT=open("diffset.dat","w")
        OUTPUT.write("data;\n")
        OUTPUT.write("set B := 1;\n")
        OUTPUT.write("end;\n")
        OUTPUT.close()
        self.model.B = DifferentialSet(bounds=(0,1))
        self.instance = self.model.create("diffset.dat")
        self.assertEqual( len(self.instance.B), 2)

if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

import logging
import sys

from coopr.core.plugin import alias
from coopr.pyomo.base import Transformation
from coopr.pyomo import *
from coopr.pyomo.base import Block
from coopr.dae import *
from coopr.dae.misc import generate_finite_elements
from coopr.dae.misc import generate_colloc_points
from coopr.dae.misc import update_diffset_indexed_component
from coopr.dae.misc import add_equality_constraints
from coopr.dae.misc import get_index_information

# If the user has numpy then the collocation points and the a matrix for
# the Runge-Kutta basis formulation will be calculated as needed. 
# If the user does not have numpy then these values will be read from a
# stored dictionary for up to 10 collocation points.
try:
    import numpy
    numpy_available=True
except ImportError:
    numpy_available=False
    from coopr.dae.utilities import *

logger = logging.getLogger('coopr.pyomo')

def factorial(n):
    if n < 0 or not isinstance(n,int):
        raise ValueError("Can only take the factorial of a non-negative integer")
    elif n > 0:
        return n*factorial(n-1)
    else:
        return 1

def conv(a,b):
    if len(a)==0 or len(b)==0:
        raise ValueError("Cannot convolve an empty list")

    ans = []
    m = len(a)
    n = len(b)

    for k in range(m+n-1):
        val = 0
        j = max(0,k-n)
        stop = min(k,m)
        while j<=stop:
             if j<m and (k-j)<n:
                val += a[j]*b[k-j]
             j += 1
        ans.insert(k,val)

    return ans

def calc_cp(alpha,beta,k):
    gamma = []
    for i in range(k+1):
        num = factorial(alpha+k)*factorial(alpha+beta+k+i)
        denom = factorial(alpha+i)*factorial(k-i)*factorial(i)
        gamma.insert(i,num/denom)

    poly = []
    for i in range(k+1):
        if i == 0:
            poly.insert(i,gamma[i])
        else:
            prod = [1]
            j=1
            while j<=i:
                prod=conv(prod,[1,-1])
                j=j+1
            while len(poly)<len(prod):
                poly.insert(0,0)
            prod = [gamma[i]*t for t in prod]
            poly = [sum(pair) for pair in zip(poly,prod)]

    cp = numpy.roots(poly)
    return cp

def calc_omega(cp):
    a=[]
    for i in range(len(cp)):
        ptmp = []
        tmp = 0
        for j in range(len(cp)):
            if j != i:
                row = []
                row.insert(0,1/(cp[i]-cp[j]))
                row.insert(1,-cp[j]/(cp[i]-cp[j]))
                ptmp.insert(tmp,row)
                tmp += 1
        p=[1]
        for j in range(len(cp)-1):
            p = conv(p,ptmp[j])
        pint = numpy.polyint(p)
        arow = []
        for j in range(len(cp)):
            arow.append(numpy.polyval(pint,cp[j]))
        a.append(arow)
    return a

class Collocation_Discretization_Transformation(Transformation):

    alias('dae.collocation_discretization', doc="TODO")

    def __init__(self):
        super(Collocation_Discretization_Transformation, self).__init__()

    def _setup(self, instance):
        instance = instance.clone()
        instance.concrete_mode()
        # Note: all_blocks is a generator and since we are changing the
        # type of Differential components to Block after the discretization,
        # we have to explicitly form the list of all Blocks before iterating
        # over them. Otherwise, the differential components will be included
        # in the iteration and _transformBlock will be called unintentionally 
        # for each differential component after their type has been changed.
        self._blocks = list(instance.all_blocks())
        self._ncp = {}
        self._nfe = {}
        self._a = {}
        self._tau = {}
        self._reduced_cp = {}

        # This is a single attribute of the discretization object because I don't think
        # you'd ever want to mix radau and legendre collocation schemes on a single
        # model. If this is a bad assumption _radau would be changed to a dictionary
        # similar to _a or _tau
        # TODO: Add option of doing Legendre collocation which will require some fancy 
        # interpolating, 'type' will then become a keyword argument for init with the 
        # possible values of 'radau' or 'legendre'.
        self._radau = True
        return instance
        
    def _get_radau_constants(self,currentds):
        """
        This function sets the radau collocation points and a values depending
        on how many collocation points have been specified and whether or not
        the user has numpy
        """
        if not numpy_available:
            if self._ncp[currentds] > 10:
                raise ValueError("Numpy was not found so the maximum number of "\
                    "collocation points is 10")
            self._tau[currentds] = radau_tau_dict[self._ncp[currentds]]
            self._a[currentds] = radau_a_dict[self._ncp[currentds]]
        else:
            alpha = 1
            beta = 0
            k = self._ncp[currentds]-1
            cp = sorted(list(calc_cp(alpha,beta,k)))
            cp.append(1.0)
            a = calc_omega(cp)
            self._tau[currentds] = cp
            self._a[currentds] = a

    def _get_legendre_constants(self,currentds):
        """
        This function sets the legendre collocation points and a values depending
        on how many collocation points have been specified and whether or not
        the user has numpy
        """
        if not numpy_available:
            if self._ncp[currentds] > 10:
                raise ValueError("Numpy was not found so the maximum number of "\
                    "collocation points is 10")
            self._tau[currentds] = legendre_tau_dict[self._ncp[currentds]]
            self._a[currentds] = legendre_a_dict[self._ncp[currentds]]
        else:
            alpha = 0
            beta = 0
            k = self._ncp[currentds]
            # TODO: finish this

    def apply(self, instance, **kwds):
        instance = self._setup(instance)
        tmpnfe = kwds.pop('nfe',10)
        tmpncp = kwds.pop('ncp',3)        
        tmpds = kwds.pop('diffset',None)

        if tmpds is not None and tmpds.type() is not DifferentialSet:
            raise TypeError("The component specified using the 'diffset' keyword "\
                "must be a differential set")
        if tmpnfe <=0:
            raise ValueError("The number of finite elements must be at least 1")
        if tmpncp <= 0:
            raise ValueError("The number of collocation points must be at least 1")
        
        if self._nfe.has_key(None):
            raise ValueError("A general discretization scheme has already been applied to "\
                    "to every differential set in the model. If you would like to specify a "\
                    "specific discretization scheme for one of the differential sets you must discretize "\
                    "each differential set individually. If you would like to apply a different "\
                    "discretization scheme to all differential sets you must declare a new Collocation"\
                    "_Discretization object")
        
        if len(self._nfe) == 0 and tmpds is None:
            # Same discretization on all differentialsets
            self._nfe[None] = tmpnfe
            self._ncp[None] = tmpncp
            currentds = None
        elif self._nfe.has_key(tmpds.name):
            raise ValueError("A discretization scheme has already been specified "\
                    "for differentialset '%s'" %s(tmpds.name))
        else :
            self._nfe[tmpds.name]=tmpnfe
            self._ncp[tmpds.name]=tmpncp
            currentds = tmpds.name
              
        if self._radau:
            self._get_radau_constants(currentds)
        else:
            self._get_legendre_constants(currentds)

        for block in self._blocks:
            self._transformBlock(block,currentds)

        # Taken from bigm
        # REQUIRED: re-call preprocess()
        instance.preprocess()
        return instance

    def _transformBlock(self, block, currentds):
        
        self._fe = {}
        for ds in block.components(DifferentialSet).itervalues():
            if currentds is None or currentds is ds.name:
                generate_finite_elements(ds,self._nfe[currentds])
                if not ds.get_changed():
                    if len(ds)-1 > self._nfe[currentds]:
                        print("***WARNING: More finite elements were found in differentialset "\
                            "'%s' than the number of finite elements specified in apply. "\
                            "The larger number of finite elements will be used." %s(ds.name))
                
                self._nfe[ds.name]=len(ds)-1
                self._fe[ds.name]=sorted(ds)
                #ds._fe=sorted(ds)
                generate_colloc_points(ds,self._tau[currentds],self._radau)
                # Adding discretization information to the differentialset object itself
                # so that it can be accessed outside of the discretization object
                disc_info = ds.get_discretization_info()
                disc_info['nfe']=self._nfe[ds.name]
                disc_info['ncp']=self._ncp[currentds]
                disc_info['tau_points']=self._tau[currentds]
                if self._radau:
                    disc_info['scheme'] = 'Radau Collocation'
                else:
                    disc_info['scheme'] = 'Legendre Collocation'
                
        
        for c in block.components().itervalues():
            if c.type() != Differential:
                update_diffset_indexed_component(c)
            else:
                update_diffset_indexed_component(c._lhs_var)

        for d in block.components(Differential).itervalues():
            if currentds is None or d.get_differentialset().name is currentds:
                add_equality_constraints(d)
                self._radau_transform(d,currentds)
                block.reclassify_subcomponent_type(d,Block)

    def _radau_transform(self,diff,currentds):
        # generate the constraints unique to this discretization scheme
        # The needed constraints are:
        # 1) Z_ik = Z_(i-1)+h_i*sum(a_kj*Zdot_ij for j in K), k = 1,...,K
        # Note: because the index specified by giving the finite element
        # and collocation point is directly related to an index in the 
        # differentialset, no additional continuity equations are required.
        ds = diff.get_differentialset()
        dv = diff.get_diffvar()
        ncp = self._ncp[currentds]
        a = self._a[currentds]
        fe = sorted(self._fe[ds.name])

        info = get_index_information(dv,ds)
        tmpidx = info['non_ds']
        idx = info['index function']

        # Iterate over non_ds indices
        for n in tmpidx:
            # Iterate over finite elements
            for i in xrange(0,len(fe)-1):
                # Iterate over collocation points
                for k in xrange(1,ncp+1):
                    # index in a starts at 0 which is why we subtract 1
                    diff._cons.add(dv[idx(n,i,k)]==dv[idx(n,i,0)] \
                                       + (fe[i+1]-fe[i])*sum(a[j-1][k-1]*diff[idx(n,i,j)] for j in xrange(1,ncp+1)))

    def _get_idx(self,l,t,n,i,k):
        """
        This function returns the appropriate index for the differential
        and the derivative variables. It's needed because the collocation 
        constraints are indexed by finite element and collocation point
        however a differentialset contains a list of all the discretization
        points and is not separated into finite elements and collocation
        points.
        """
        
        tmp = t.index(t._fe[i])
        tik = t[tmp+k]
        if n is None:
            return tik
        else:
            tmpn=n
            if not isinstance(n,tuple):
                tmpn = (n,)
            return tmpn[0:l]+(tik,)+tmpn[l:]
          

    def reduce_collocation_points(self, instance, var=None, ncp=None, diffset=None):
        """
        This method will add additional constraints to a model if some
        of the Variables are specified as having less collocation points
        than the default
        """
        if diffset is None:
            raise TypeError("A differential set must be specified")
        if diffset.type() is not DifferentialSet:
            raise TypeError("The component specified using the 'diffset' keyword "\
                "must be a differential set")
        ds = instance.find_component(diffset.name)
        if ds is None:
            raise ValueError("DifferentialSet '%s' is not a valid component of the discretized "\
                "model instance" %(diffset.name))

        if len(self._ncp) == 0:
            raise RuntimeError("This method should only be called after using the apply() method "\
                "to discretize the model")
        elif self._ncp.has_key(None):
            tot_ncp = self._ncp[None]
        elif self._ncp.has_key(ds.name):
            tot_ncp = self._ncp[ds.name]
        else:
            raise ValueError("DifferentialSet '%s' has not been discretized yet, please call "\
                "the apply() method with this DifferentialSet to discretize it before calling "\
                "this method" %s(ds.name))

        if var is None:
            raise TypeError("A variable must be specified")
        if var.type() is not Var:
            raise TypeError("The component specified using the 'var' keyword "\
                "must be a variable")
        tmpvar = instance.find_component(var.name)
        if tmpvar is None:
            raise ValueError("Variable '%s' is not a valid component of the discretized "\
                "model instance" %(var.name))

        var = tmpvar

        if ncp is None:
            raise TypeError("The number of collocation points must be specified")
        if ncp <= 0:
            raise ValueError("The number of collocation points must be at least 1")
        if ncp > tot_ncp:
            raise ValueError("The number of collocation points used to interpolate "\
                "an individual variable must be less than the number used to discretize "\
                "the original model")
        if ncp == tot_ncp:
            # Nothing to be done
            return instance

        # Check to see if the differentialset is an indexing set of the variable
        if var.dim() == 1:
            if ds not in var._index:
                raise IndexError("DifferentialSet '%s' is not an indexing set of the variable '%s'"\
                    % (ds.name,var.name))
        elif ds not in var._index_set:
            raise IndexError("DifferentialSet '%s' is not an indexing set of the variable '%s'"\
                % (ds.name,var.name))

        if self._reduced_cp.has_key(var.name):
            temp = self._reduced_cp[var.name]
            if temp.has_key(ds.name):
                raise RuntimeError("Variable '%s' has already been constrained to a reduced "\
                    "number of collocation points over DifferentialSet '%s'.")
            else:
                temp[ds.name]=ncp
        else:
            self._reduced_cp[var.name] = {ds.name:ncp}                
        
        list_name = var.name+"_interpolation_constraints"
        
        instance.add_component(list_name,ConstraintList())
        conlist = instance.find_component(list_name)

        t = sorted(ds)
        fe = ds._fe
        info = get_index_information(var,ds)
        tmpidx = info['non_ds']
        idx = info['index function']

        # Iterate over non_ds indices
        for n in tmpidx:
            # Iterate over finite elements
            for i in xrange(0,len(fe)-1):
                # Iterate over collocation points
                for k in xrange(1,tot_ncp-ncp+1):
                    if ncp == 1:
                        # Constant over each finite element
                        conlist.add(var[idx(n,i,k)]==var[idx(n,i,tot_ncp)])
                    else:
                        tmp = t.index(fe[i])
                        tmp2 = t.index(fe[i+1])
                        ti = t[tmp+k]
                        tfit = t[tmp2-ncp+1:tmp2+1]
                        coeff = self._interpolation_coeffs(ti,tfit)
                        conlist.add(var[idx(n,i,k)]== sum(var[idx(n,i,j)]*coeff.next() for j in xrange(tot_ncp-ncp+1,tot_ncp+1)))

        return instance

    def _interpolation_coeffs(self,ti,tfit):
  
        for i in tfit:
            l=1
            for j in tfit:
                if i != j:
                    l = l*(ti-j)/(i-j)
            yield l




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Differential']

import sys
import types
import weakref
import logging

from coopr.pyomo import *
from coopr.pyomo.base.sets import *
from coopr.pyomo.base.misc import create_name, apply_indexed_rule
from coopr.dae import DifferentialSet
from coopr.pyomo.base.block import SimpleBlock
from coopr.pyomo.base.constraint import Constraint
from coopr.pyomo.base.var import Var

logger = logging.getLogger('coopr.pyomo')


class Differential(SimpleBlock):
    """
    An object that defines a differential equation
    
    keyword arguments:
    name: name of this object
    rule: a function or expression describing the right hand side
          of the differential equation
    expr: same as rule
    doc: documentation string for the differential equation
    dv: The variable being differentiated
    dvar: same as dv
    ds: The Differential Set over which the derivative of the differential
          variable is taken
    dset: same as ds
    bounds: A tuple or function setting bounds on the derivative of the
          differential variable

    Class Attributes:
        _ds_argindex: An integer representing where the differentialset index is located 
             in the order of all the indices
        _non_ds: The indexing sets that are not the differential set
        _lhs_var: Variable representing the derivative of the differential variable
        _cons: Constraints that are added during discretization
  
    """
  
    def __init__(self, *args, **kwd):
        if args != ():
            raise ValueError("A Differential should not have any positional arguments, "\
                                 "the indexing of a Differential is exactly the indexing "\
                                 "of the differential variable")     

        if "rule" in kwd and "expr" in kwd:
            raise TypeError("Cannot specify both 'rule' and 'expr' " + \
                "keywords in a Differential")
        if "dv" in kwd and "dvar" in kwd:
            raise TypeError("Cannot specify both 'dv' and 'diffvar' " +\
                "keywords in a Differential")
        if "ds" in kwd and "dset" in kwd:
            raise TypeError("Cannot specify both 'ds' and 'differentialset' " +\
                "keywords in a Differential")
        
        tmprule = kwd.pop("rule", None)
        tmprule = kwd.pop("expr", tmprule)
        if tmprule is None:
            raise TypeError("A rule must be specified for a Differential")
        elif not isinstance(tmprule,types.FunctionType):
            # Because a Differential must be indexed by at least a DifferentialSet the rule
            # supplied must be a function
            raise TypeError("A Differential rule must be a function")

        tmpdv = kwd.pop("dv", None)
        tmpdv = kwd.pop("dvar", tmpdv)
        if tmpdv is None:
            raise TypeError("A differential variable must be specified")
        elif not isinstance(tmpdv,Var):
            raise TypeError("A differential variable must be a variable")
        tmpbounds = kwd.pop("bounds",None)
        tmpinitial = kwd.pop("initialize",None)
        
        tmpds = kwd.pop("ds",None)
        tmpds = kwd.pop("dset",tmpds)
        if tmpds is not None:
            if not isinstance(tmpds,DifferentialSet):
                raise TypeError("The component specified as a differentialset is not "\
                    "a differentialset")

        # Check to make sure that a differential set specified by a keyword 
        # argument belongs to the indexing sets of the differential variable
        if not tmpds is None:
            if tmpdv.dim() == 1:
                if not tmpds in tmpdv._implicit_subsets:
                    raise TypeError("DifferentialSet '%s' is not an indexing set of the differential variable"\
                        % str(tmpds))
            elif not tmpds in tmpdv._implicit_subsets:
                raise TypeError("DifferentialSet '%s' is not an indexing set of the differential variable"\
                    % str(tmpds))
        
        # Find the index of the DifferentialSet in the differential variable. If
        # the differential variable is indexed by mulitple DifferentialSets check 
        # to make sure the DifferentialSet was set using a keyword argument. Also,
        # generate an indexing argument list which contains all the indicies
        # of the differential variable except for the differential set associated 
        # with this differential. DifferentialSets must be *Explicit* indexing sets
        # of the differential variable

        indargs=[]
        dsCount=0
        dsindex=0
        tmpds2=None

        if tmpdv.dim() == 1:
            if isinstance(tmpdv._index,DifferentialSet):
                dsCount = 1
                if tmpds is None:
                    tmpds = tmpdv._index
            else:
                raise IndexError("A differential variable must be indexed by a differential set")
        else:
            # If/when Var is changed to SparseIndexedComponent, the _index 
            # attribute below may need to be changed
            indCount = 0
            for index in tmpdv._implicit_subsets:
                if isinstance(index,DifferentialSet):
                    if not tmpds is None:
                        if index == tmpds:
                            dsindex = indCount
                        else:
                            indargs.append(index)  # If dv is indexed by multiple differentialsets treat
                                                   # other differentialsets like a normal idexing set
                    else:
                        tmpds2=index
                        dsindex=indCount
                    indCount += 1     # A differentialset must be one dimensional
                    dsCount += 1
                else:
                    indargs.append(index)
                    indCount += index.dimen
        
        if dsCount == 0:
            raise IndexError("Differential variable must be indexed by a DifferentialSet")
        elif dsCount == 1:
            if tmpds is None:
                tmpds = tmpds2
        else:
            if tmpds is None:
                raise TypeError("If a differential variable is indexed by multiple differentialsets "\
                    "the desired differentialset must be specified using a keyword argument")
        
        kwd.setdefault('ctype', Differential)
        
        SimpleBlock.__init__(self,**kwd)
     
        self._rule = tmprule
        self._bounds = tmpbounds
        self._initial = tmpinitial
        self._ds_argindex = dsindex
             
        # __setattr__ in block handles things derived from
        # component strangely. If we use it to add the three 
        # attributes below it will actually change their names
        # which isn't the desired behavior. This is following
        # something similar in Piecewise
        self.__dict__['_dv'] = tmpdv
        self.__dict__['_ds'] = tmpds


        if indargs == []:
            self.__dict__['_non_ds'] = (None,)
        elif len(indargs)>1:
            self.__dict__['_non_ds']=tuple(a for a in indargs)
        else:
            self.__dict__['_non_ds']=(indargs[0],)


    def __str__(self):
        """
        Return a string representation of the differential
        """
        if self.name is None:
            return ""
        else:
            return self.name
    
    def construct(self, data=None):
        """
        Creates derivative variables for the indices in the DifferentialSet which 
        have been specified before the discretization
        """
        generate_debug_messages = (__debug__ is True) and (logger.isEnabledFor(logging.DEBUG) is True)
        if generate_debug_messages is True:
            logger.debug("Constructing differential '%s'",self.name)

        if self._constructed:
            return
        self._constructed = True
        self._defer_construction = False

        # Not sure if the following line is needed
        #self.concrete_mode()

        if type(self._initial) is types.FunctionType:
            # If a function is supplied to 'initialize' we modify it slightly
            # to ensure that the parent block for the differential gets passed
            # to the function and not the differential itself
            def _init(d,*args):
                return self._initial(self._parent(),*args)
            init=_init
        else:
            init=self._initial

        if type(self._bounds) is types.FunctionType:
            # If a function is supplied to 'bounds' we modify it slightly 
            # to ensure that the parent block for the differential gets passed 
            # to the function and not the differential itself
            def _dbounds(d,*args):
                return self._bounds(self._parent(),*args)
            bds = _dbounds
        else:
            bds = self._bounds
        
        # TODO: This will need to be changed when Var is moved to SparseIndexedComponent
        if self._dv.dim()==1: 
            self.add_component('_lhs_var',Var(self._dv._index,bounds=bds,initialize=init))
        else:
            self.add_component('_lhs_var',Var(self._dv._index,bounds=bds,initialize=init))

        self.add_component('_cons',ConstraintList())
            
    def __getitem__(self,indx):

        if indx is None:
            return self
        if indx in self._lhs_var.keys():
            return self._lhs_var[indx]
        else:
            raise KeyError("Unknown index in Differential '%s': %s" % (self.name,str(indx)))

    def get_diffvar(self):
        return self._dv

    def get_differentialset(self):
        return self._ds

    def pprint(self, ostream=None, verbose=False, prefix=""):
        # TBD: Do something with the prefix input argument!
        if ostream is None:
            ostream = sys.stdout
        ostream.write("  %s : " %(self.name,))
        ostream.write("DiffVar=%s \tDifferentialSet=%s\n" %(self._dv.name,self._ds.name))
        if not self._constructed:
            ostream.write("\tNot Constructed\n")
            return
        else:
            self._lhs_var.pprint()
            self._cons.pprint()

register_component(Differential, "Differential equation expression in a model.")



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

import logging
import sys
import types
import pyutilib.math
from coopr.pyomo import *
from coopr.dae import *
from coopr.pyomo.base.sparse_indexed_component import *
from coopr.pyomo.base.misc import apply_indexed_rule

logger = logging.getLogger('coopr.pyomo')

def generate_finite_elements(ds,nfe):
    """ 
    This function first checks to see if the number of finite elements
    in the differential set is equal to nfe. If the number of finite
    elements is less than nfe, additional points will be generated. If
    the number of finite elements is greater than or equal to nfe the 
    differential set will not be modified
    """
    if (len(ds)-1) >= nfe:
        # In this case the differentialset already contains the
        # desired number or more than the desired number of finite
        # elements so no additional points are needed.
        return     
    elif len(ds) == 2:
        # If only bounds have been specified on the differentialset we
        # generate the desired number of finite elements by 
        # spreading them evenly over the interval
        step = (max(ds)-min(ds))/float(nfe)
        tmp = min(ds)+step
        while tmp<=(max(ds)-step):
            ds.add(tmp)
            tmp+=step
        ds.set_changed(True)
        ds._fe = sorted(ds)
        return
    else:
        # This is the case where some points have been specified
        # inside of the bounds however the desired number of finite 
        # elements has not been met. We first look at the step sizes
        # between the existing points. Then an additional point
        # is placed at the midpoint of the largest step. This
        # process is repeated until we have achieved the desired 
        # number of finite elements. If there are multiple "largest steps"
        # the point will be placed at the first occurance of the
        # largest step

        addpts = nfe-(len(ds)-1)
        while addpts>0:
            _add_point(ds)
            addpts -= 1
        ds.set_changed(True)
        ds._fe = sorted(ds)
        return

def _add_point(ds):
    sortds = sorted(ds)
    maxstep = sortds[1]-sortds[0]
    maxloc = 0
    for i in range(2,len(sortds)):
        if (sortds[i]-sortds[i-1])>maxstep:
            maxstep = sortds[i]-sortds[i-1]
            maxloc = i-1
            
    ds.add(sortds[maxloc]+maxstep/2.0)

def generate_colloc_points(ds,tau,radau):
    """
    This function adds collocation points between the finite elements
    in the differential set
    """
    if len(tau)==1 and radau:
        # Radau collocation has a collocation point at the finite
        # element locations so no need to add additional points
        return
    ds.set_changed(True)
    fes = sorted(ds)
    for i in range(1,len(fes)):
        h = fes[i]-fes[i-1]
        for j in range(len(tau)):
            pt = fes[i-1]+h*tau[j]
            if pt not in ds:
                ds.add(pt)

def update_diffset_indexed_component(comp):
    """
    Update any model components other than Differential
    which are indexed by a DifferentialSet that has changed
    """
    # FIXME: This implementation is a hack until Var and Constraint get
    # moved over to Sparse_Indexed_Component. The update methods below are 
    # roughly what the '_default' method in Var and Constraint should
    # do when they get reimplemented.

    # This implementation also assumes that only Var and Constraint components
    # will be explicitly indexed by a DifferentialSet and thus only checks for 
    # these two components. 

    # Additionally, this implemenation will *NOT* check
    # for or update components which use a DifferentialSet implicitly. ex) an objective
    # function which iterates through a DifferentialSet and sums the squared error. 
    # If you use a DifferentialSet implicitly you must initialize it with every
    # index you would like to have access to!

    if comp.type() is Suffix:
        return
    if comp.dim() == 1:
        if comp._index.type() == DifferentialSet: 
            if comp._index.get_changed():
                if comp.type() == Var:
                    _update_var(comp)
                elif comp.type() == Constraint:
                    _update_constraint(comp)
    elif comp.dim() > 1:
        if isinstance(comp,SparseIndexedComponent):
            indexset = comp._implicit_subsets
        else:
            indexset = comp._index_set

        for s in indexset:
            if s.type() == DifferentialSet and s.get_changed:
                if comp.type() == Var:
                    _update_var(comp)
                elif comp.type() == Constraint:
                    _update_constraint(comp)
                
                    
def _update_var(v):
    """
    This method will construct any additional indices in a variable 
    resulting from the discretization of DifferentialSet 
    """

    # Note: This is not required it is handled by the _default method on
    #       Var (which is now a SparseIndexedComponent). However, it
    #       would be much slower to rely on that method to generate new
    #       _VarData for a large number of new indices.
    new_indices = set(v._index)-set(v._data.keys())
    v._add_members(new_indices)
    v._initialize_members(new_indices)

def _update_constraint(con):
    """
    This method will construct any additional indices in a constraint
    resulting from the discretization.
    """

    # FIXME: This isn't quite as hackish as _update_constraint but it
    # would be nice to move this "update" operation to the contraint
    # object itself

    for i in con.index_set():
        if i not in con.keys():
            # Code taken from the construct() method of Constraint
            _rule=con.rule
            _parent=con._parent()
            con.add(i,apply_indexed_rule(con,_rule,_parent,i))

def add_equality_constraints(diff):
    """
    This function generates the equality constraints that set the 
    derivative at each index of the differential variable equal to
    the expression generated using the rule supplied for the rhs when 
    the Differential block was declared. This equality constraint should
    not be enforced at the initial point in the DifferentialSet.
    """

    # NOTE!!! The differential equation is currently not being
    # enforced at the first point in the differentialset (i.e. time zero).
    # This could lead to major issues if a user tries to specify an
    # initial condition for a derivative at time zero. If var was a
    # sparse_indexed_component we could deal with this
    ds = diff.get_differentialset()
    start = ds._fe[0]
    for i in diff.get_diffvar().keys():
        if isinstance(i,tuple):
            if i[diff._ds_argindex] == start:
                continue
            expr = diff._rule(diff._parent(),*i)
        else:
            if i == start:
                continue
            expr = diff._rule(diff._parent(),i)
        diff._cons.add(diff[i]==expr)

def get_index_information(var,ds):
    """
    This method will find the index location of the ds in the var, return
    a list of the non_ds indices and return a function that can be used to
    access specific indices in var indexed by a DifferentialSet by specifying the
    finite elemtent and collocation point. Users of this method should have
    already confirmed that ds is an indexing set of var and that it's a DifferentialSet
    """

    # Find index order of differentialset in the variable
    indargs=[]
    dsindex=0
    tmpds2=None

    if var.dim() != 1:
        # If/when Var is changed to SparseIndexedComponent, the _index_set 
        # attribute below may need to be changed
        indCount = 0
        for index in var._index_set:
            if isinstance(index,DifferentialSet):
                if index ==ds:
                    dsindex = indCount
                else:
                    indargs.append(index)  # If var is indexed by multiple differentialsets treat
                                           # other differentialsets like a normal idexing set
                indCount += 1     # A differentialset must be one dimensional
            else:
                indargs.append(index)
                indCount += index.dimen

    if indargs == []:
        non_ds = (None,)
    elif len(indargs)>1:
        non_ds = tuple(a for a in indargs)
    else:
        non_ds = (indargs[0],)

    if None in non_ds:
        tmpidx = (None,)
    elif len(non_ds)==1:
        tmpidx = non_ds[0]
    else:
        tmpidx = non_ds[0].cross(*non_ds[1:])

    # Lambda function used to generate the desired index
    # more concisely
    idx = lambda n,i,k: _get_idx(dsindex,ds,n,i,k)

    info = {}
    info['non_ds']=tmpidx
    info['index function'] = idx
    return info


def _get_idx(l,ds,n,i,k):
    """
    This function returns the appropriate index for a variable 
    indexed by a differential set. It's needed because the collocation 
    constraints are indexed by finite element and collocation point
    however a differentialset contains a list of all the discretization
    points and is not separated into finite elements and collocation
    points.
    """
    t = sorted(ds)
    tmp = t.index(ds._fe[i])
    tik = t[tmp+k]
    if n is None:
        return tik
    else:
        tmpn=n
        if not isinstance(n,tuple):
            tmpn = (n,)
    return tmpn[0:l]+(tik,)+tmpn[l:]






#
# Unit Tests for the Differential() Object
#

import os
import sys
from os.path import abspath, dirname
currdir = dirname(abspath(__file__))+os.sep

import coopr.environ
from coopr.pyomo import *
from coopr.dae import *
import pyutilib.th as unittest

class TestDifferential(unittest.TestCase):
    
    def setUp(self):
        self.model = ConcreteModel()
        m = self.model
        m.t1 = DifferentialSet(bounds=(0,1))
        m.t2 = DifferentialSet(initialize=[2,4,6])
        m.s = Set(initialize=[1,2,3])
        m.s2 = Set(initialize=[(1,1),(2,2),(3,3)])

        m.v1 = Var(m.t1)
        m.v2 = Var(m.s,m.t1)
        m.v3 = Var(m.t1,m.s,m.t2)
        m.v4 = Var(m.s2,m.t1,m.s)

    # test __init__
    def test_init(self):
        m = self.model
        def _vdot1(model,ti):
            return model.v1[ti]
        def _vdot2(model,si,ti):
            return model.v2[si,ti]
        def _vdot3(model,t1i,si,t2i):
            return model.v3[t1i,si,t2i]
        def _vdot4(model,i,j,ti,si):
            return model.v4[i,j,ti,si]

        def _vdot2_bounds(model,si,ti):
            return (0,si)

        m.vdot1 = Differential(dvar=m.v1,expr=_vdot1)
        m.vdot2 = Differential(dv=m.v2,rule=_vdot2)
        m.vdot3 = Differential(dv=m.v3,rule=_vdot3,ds=m.t2)
        m.vdot4 = Differential(dv=m.v4,rule=_vdot4)
        
        del m.vdot1
        del m.vdot2
        del m.vdot3

        m.vdot1 = Differential(dv=m.v1,rule=_vdot1, bounds=(0,10), initialize=2)
        m.vdot2 = Differential(dv=m.v2,ds=m.t1, \
                                       bounds=_vdot2_bounds,rule=_vdot2)
        m.vdot3 = Differential(dvar=m.v3,expr=_vdot3,\
                                       bounds=(-10,10),dset=m.t1)
        

    # test bad keyword arguments
    def test_bad_kwds(self):
        m = self.model
                
        def _vdot(model,ti):
            return model.v1[ti]
        
        try:
            m.vdot = Differential(dv=m.v1,rule=_vdot,expr=_vdot)
            self.fail("Expected TypeError because both rule and expr are given as keyword arguments")
        except TypeError:
            pass

        try:
            m.vdot = Differential(dv=m.v1,dvar=m.v1,rule=_vdot)
            self.fail("Expected TypeError because both dv and dvar are given as keyword arguments")
        except TypeError:
            pass

        try:
            m.vdot = Differential(dv=m.v1,rule=_vdot,ds=m.t1,dset=m.t1)
            self.fail("Expected TypeError because both ds and dset are given as keyword arguments")
        except TypeError:
            pass
        

    # test valid declarations
    def test_valid_declaration(self):
        m = self.model
        m.bound_vdot2 = Param(m.s,m.t1,default=5)
        m.init_vdot2 = Param(m.t1,default=3)

        def _vdot1(model,ti):
            return model.v1[ti]
        def _vdot2(model,si,ti):
            return model.v2[si,ti]
        def _vdot3(model,t1i,si,t2i):
            return model.v3[t1i,si,t2i]
        def _vdot4(model,i,j,ti,si):
            return model.v4[i,j,ti,si]

        def _vdot2_bounds(model,si,ti):
            return (0,model.bound_vdot2[si,ti])             

        def _vdot2_init(model,si,ti):
            return model.init_vdot2[ti]

        m.vdot1 = Differential(dv=m.v1,rule=_vdot1, bounds=(0,10),initialize=2)
        m.vdot2 = Differential(dv=m.v2,ds=m.t1, \
                                       bounds=_vdot2_bounds,rule=_vdot2, initialize=_vdot2_init)
        m.vdot3 = Differential(dvar=m.v3,expr=_vdot3,\
                                       bounds=(-10,10),dset=m.t1)
        m.vdot4 = Differential(dv=m.v4,rule=_vdot4)

        self.assertTrue(None in m.vdot1._non_ds)
        self.assertTrue(m.s in m.vdot2._non_ds)
        self.assertTrue(m.t2 in m.vdot3._non_ds)
        self.assertTrue(m.s in m.vdot3._non_ds)

        self.assertTrue(m.t1 == m.vdot1._ds)
        self.assertTrue(m.t1 == m.vdot2._ds)
        self.assertTrue(m.t1 == m.vdot3._ds)
        
        self.assertTrue(m.v1 == m.vdot1._dv)
        self.assertTrue(m.v2 == m.vdot2._dv)
        self.assertTrue(m.v3 == m.vdot3._dv)

        self.assertTrue(m.vdot1._ds_argindex == 0)
        self.assertTrue(m.vdot2._ds_argindex == 1)
        self.assertTrue(m.vdot3._ds_argindex == 0)
        self.assertTrue(m.vdot4._ds_argindex == 2)

        self.assertTrue(str(m.vdot1) is 'vdot1')
        self.assertTrue(m.vdot1.name is 'vdot1')

        self.assertEqual(m.vdot1[1].value, 2)
        self.assertEqual(m.vdot2[1,0].value, 3)
        self.assertEqual(m.vdot3[0,1,2].value,None)
        
    # test invalid declarations
    def test_invalid_declaration(self):
        m = self.model
        m.tmp = Var(m.s)
        m.tmp2 = Var(m.s,m.s)
        
        def _tmpdot(model,si):
            return model.tmp[si]
        def _tmp2dot(model,i,j):
            return model.tmp2[i,j]
        def _vdot1(model,ti):
            return model.v1[ti]
        def _vdot2(model,si,ti):
            return model.v2[si,ti]
        def _vdot3(model,t1i,si,t2i):
            return model.v3[t1i,si,t2i]

        try:
            m.vdot2 = Differential(m.s,dv=m.v1,rule=_vdot1)
            self.fail("Expected ValueError because a positional argument was specified")
        except ValueError:
            pass

        try:
            m.vdot3 = Differential(rule=_vdot3)
            self.fail("Expected TypeError because no differential variable specified")
        except TypeError:
            pass

        try:
            m.vdot2 = Differential(dv=m.v2)
            self.fail("Expected TypeError because no rule was specified")
        except TypeError:
            pass

        try:
            m.tmpdot = Differential(dv=m.tmp,rule=_tmpdot)
            self.fail("Expected IndexError because the diffvar isn't indexed by a differentialset")
        except IndexError:
            pass

        try:
            m.tmp2dot = Differential(dv=m.tmp2,rule=_tmp2dot)
            self.fail("Expected IndexError because the diffvar isn't indexed by a differentialset")
        except IndexError:
            pass

        try:
            m.vdot2 = Differential(dv=m.v2,ds=m.t2,rule=_vdot2)
            self.fail("Expected TypeError because the specified differentialset is not"\
                          " an indexing set of the diffvar")
        except TypeError:
            pass

        try:
            m.vdot = Differential(dv=m.v1,ds=m.t2,rule=_vdot1)
            self.fail("Expected TypeError because the specified differentialset is not"\
                          " an indexing set of the diffvar")
        except TypeError:
            pass

        try:
            m.vdot3 = Differential(dv=m.v3,rule=_vdot3)
            self.fail("Expected TypeError because a differentialset was not specified")
        except TypeError:
            pass

    # test when keyword arguments with the wrong type are specified
    def test_badtype_kwds(self):
        m = self.model

        def _vdot1(model,ti):
            return model.v1[ti]
        def _vdot2(model,si,ti):
            return model.v2[si,ti]
        def _vdot3(model,t1i,si,t2i):
            return model.v3[t1i,si,t2i]

        try:
            m.vdot2 = Differential(dv=m.v2,rule=m.v2)
            self.fail("Expected TypeError becasue the rule is not a function")
        except TypeError:
            pass

        try:
            m.vdot2 = Differential(dv=m.t2,rule=_vdot2)
            self.fail("Expected TypeError because the specified differential" \
                          " variable is not a variable")
        except TypeError:
            pass

        try:
            m.vdot2 = Differential(dv=m.v2,rule=_vdot2,ds=m.s)
            self.fail("Expected TypeErrpr because the specified differential "\
                          "set is not a DifferentialSet component")
        except TypeError:
            pass

    # test __str__
    def test_str(self):
        m = self.model
        def _vdot1(model,ti):
            return model.v1[ti]
        m.vdot1 = Differential(dv=m.v1,rule=_vdot1, bounds=(0,10))
        self.assertTrue(m.vdot1.__str__() == "vdot1")

    # test pprint()
    def test_pprint(self):
        m = self.model
        def _vdot1(model,ti):
            return model.v1[ti]
        def _vdot2(model,si,ti):
            return model.v2[si,ti]
        def _vdot3(model,t1i,si,t2i):
            return model.v3[t1i,si,t2i]
        def _vdot4(model,i,j,ti,si):
            return model.v4[i,j,ti,si]

        def _vdot2_bounds(model,si,ti):
            return (0,si)             

        m.vdot1 = Differential(dv=m.v1,rule=_vdot1, bounds=(0,10))
        m.vdot2 = Differential(dv=m.v2,ds=m.t1, \
                                       bounds=_vdot2_bounds,rule=_vdot2)
        m.vdot3 = Differential(dvar=m.v3,expr=_vdot3,\
                                       bounds=(-10,10),dset=m.t1)
        m.vdot4 = Differential(dv=m.v4,rule=_vdot4)

        from six.moves import StringIO
        output = StringIO()
        m.vdot1.pprint(ostream=output)
        m.vdot2.pprint(ostream=output)
        m.vdot3.pprint(ostream=output)
        m.vdot4.pprint(ostream=output)

    # test construct
    def test_construct(self):
        mod = AbstractModel()
        mod.s = Set(initialize=[1,2,3])
        mod.t = DifferentialSet(bounds=(0,1))
        mod.v = Var(mod.s,mod.t)
        def _vdot(model,i,j):
            return model.v[i,j]

        mod.vdot = Differential(dv=mod.v,rule=_vdot)
        mod.vdot.pprint()
        mod.s.construct()
        mod.t.construct()
        mod.v.construct()
        self.assertFalse(mod.vdot._constructed)
        mod.vdot.construct()
        self.assertTrue(mod.vdot._constructed)
        mod.vdot.construct()

    # test __getitem__
    def test_getitem(self):
        m = self.model
        def _vdot1(model,ti):
            return model.v1[ti]
        def _vdot2(model,si,ti):
            return model.v2[si,ti]

        m.vdot1 = Differential(dv=m.v1,rule=_vdot1, bounds=(0,10))
        m.vdot2 = Differential(dv=m.v2,ds=m.t1,rule=_vdot2)

        for i in m.v1.keys():
            try:
                tmp = m.vdot1[i]
            except KeyError:
                self.fail("A valid index should not throw a KeyError")

        for i in m.v2.keys():
            try:
                tmp = m.vdot2[i]
            except KeyError:
                self.fail("A valid index shoulf not throw a KeyError")
        
        try:
            tmp = m.vdot1[100]
            self.fail("Expected KeyError because the supplied index is not a vlid index")
        except KeyError:
            pass
                
if __name__ == "__main__":
    unittest.main()
